#include "precompile.h"

#include "entity.h"
#include "collider.h"
#include "room.h"
#include "human.h"
#include "app.h"

Entity::Entity()
{

}

Entity::~Entity()
{
    ClearColliders();
}

void Entity::Initialize()
{
    xtimer_attacher.xtimer = &room->xtimer;
}

void Entity::GetAabbBox(AabbCollider& aabb_box)
{
    aabb_box.active = true;
    aabb_box.owner = this;
}

void Entity::GetCircleBox(CircleCollider& circle_box)
{
    circle_box.active = true;
    circle_box.owner = this;
    circle_box.rad = 1;
}

bool Entity::TestCollision(Entity* b)
{
    App::Instance()->perf.test_times++;
    if (b->dead) {
        return false;
    }
    for (auto& a_collider : colliders) {
        for (auto& b_collider : b->colliders) {
            if (a_collider->Intersect(b_collider)) {
                return true;
            }
        }
    }
    return false;
}

bool Entity::TestCollision(ColliderComponent* b)
{
    if (b->owner->dead) {
        return false;
    }
    for (auto& a_collider : colliders) {
        if (a_collider->Intersect(b)) {
            return true;
        }
    }
    return false;
}

bool Entity::TestCollisionEx(const a8::Vec2& aabb_pos, AabbCollider& aabb_box)
{
    for (auto& a_collider : colliders) {
        if (a_collider->IntersectEx(aabb_pos, &aabb_box)) {
            return true;
        }
    }
    return false;
}

void Entity::ClearColliders()
{
    for (auto& itr : colliders) {
        ColliderComponent* collider = itr;
        DestoryCollider(collider);
    }
    colliders.clear();
}

void Entity::BroadcastFullState()
{
    std::set<GridCell*> grid_list;
    room->grid_service.GetAllCells(grid_id, grid_list);
    for (auto& grid : grid_list) {
        for (Human* hum : grid->human_list) {
            hum->AddToNewObjects(this);
        }
    }
}

void Entity::BroadcastDeleteState()
{
    std::set<GridCell*> grid_list;
    room->grid_service.GetAllCells(grid_id, grid_list);
    for (auto& grid : grid_list) {
        for (Human* hum : grid->human_list) {
            hum->RemoveObjects(this);
        }
    }
}

void Entity::AddCollider(ColliderComponent* collider)
{
    colliders.push_back(collider);
}
