#pragma once

#include "entity.h"
#include "cs_proto.pb.h"
#include "GGListener.h"

namespace MetaData
{
    class Snake;
}

enum HumanStatus
{
    HS_End
};

struct xtimer_list;
class CircleCollider;
class AabbCollider;
class Obstacle;
class Loot;
class Human : public Entity
{
 public:
    MetaData::Snake* meta = nullptr;
    int socket_handle = 0;
    long ip_saddr = 0;
    int team_id = 0;
    std::string account_id;
    std::string session_id;
    std::string from_appid;
    std::string team_uuid;
    bool auto_fill = false;
    int today_enter_times = 0;
    int account_registertime = 0;
    HumanAbility buff;

    a8::Vec2 move_dir;

    std::string name;
    std::string avatar_url;
    bool disconnected = false;
    long long join_frameno = 0;
    int status = 0;
    bool has_pass = 0;
    bool real_dead = false;
    xtimer_list* revive_timer = nullptr;
    int dead_times = 0;

    bool need_sync_team_data = false;
    bool need_sync_teammate_data = false;
    bool need_sync_active_player = false;

    PlayerStats stats;

    std::set<Human*>* team_members = nullptr;
    std::set<Human*> kill_humans;
    long long send_msg_times = 0;

    Human();
    virtual ~Human() override;
    virtual void Initialize() override;
    virtual void FillMFObjectPart(cs::MFObjectPart* part_data) override;
    virtual void FillMFObjectFull(cs::MFObjectFull* full_data) override;
    virtual void FillMFPlayerStats(cs::MFPlayerStats* stats);
    virtual void GetAabbBox(AabbCollider& aabb_box);
    void FillMFTeamData(cs::MFTeamData* team_data);
    void RecalcSelfCollider();
    void SyncAroundPlayers(const char* file, int line, const char* func);
    void BeKill(int killer_id, const std::string& killer_name);
    void AddToNewObjects(Entity* entity);
    void AddToPartObjects(Entity* entity);
    void RemovePartObjects(Entity* entity);
    void RemoveObjects(Entity* entity);
    void AddOutObjects(Entity* entity);
    void RemoveOutObjects(Entity* entity);
    bool HasLiveTeammate();
    void RefreshView();
    void OnGridListChange(std::set<GridCell*>& old_grid_list,
                          std::set<GridCell*>& inc_grid_list,
                          std::set<GridCell*>& dec_grid_list
                          );
    void FillMFActivePlayerData(cs::MFActivePlayerData* player_data);
    void FillMFGasData(cs::MFGasData* gas_data);
    bool CanSee(const Human* hum) const;
    void RecalcAttr();
    void RecalcBuff();
    void AddObserver(Human* observer);
    void RemoveObserver(Human* observer);
    void SendUpdateMsg();
    template <typename T>
    void SendNotifyMsg(T& msg)
    {
        GGListener::Instance()->SendToClient(socket_handle, 0, msg);
    }
    void SendGameOver();
    void FollowTarget(Human* target);
    void SendDebugMsg(const std::string& debug_msg);
    void SendRollMsg(const std::string& roll_msg);
    void SendUIUpdate();
    void SendWxVoip();

protected:
    void _UpdateMove(int speed);

private:
    void ClearFrameData();
    void GenBattleReportData(a8::MutableXObject* params);
    void FillSMGameOver(cs::SMGameOver& msg);
    void SendBattleReport();
    void OnDie();
    void Revive();
    void CreateSnake();
    void FillMFSnake(cs::MFSnake* snake_pb);

protected:
    int curr_node_id_ = 1;
    SnakeBodyNode* head_ = nullptr;
    SnakeBodyNode* tail_ = nullptr;
    std::list<SnakeBodyNode*> body_list;
    long long last_sync_teamdata_frameno_ = 0;
    bool leave_ = false;
    long long leave_frameno_ = 0;

    std::set<Entity*> new_objects;
    std::set<Entity*> part_objects;
    std::set<int> del_objects;
    std::set<int> out_objects;
    std::vector<int> emotes_;
    std::set<Human*> observers_;
    Human* follow_target_ = nullptr;
    bool follow_synced_active_player = false;

private:
    CircleCollider* self_collider_ = nullptr;
    long long last_sync_gas_frameno = 0;

    bool already_report_battle_ = false;
    bool sent_battlereport_ = false;
    bool sent_game_end_ = false;

    friend class FrameMaker;
    friend class FrameEvent;
};
