#include "precompile.h"

#include <a8/stringlist.h>
#include <a8/csvreader.h>

#include "framework/cpp/utils.h"

#include "metamgr.h"
#include "app.h"

class MetaDataLoader
{
public:
    std::list<metatable::Parameter> parameter_meta_list;
    std::list<MetaData::Parameter> parameter_list;
    std::list<metatable::Map> map_meta_list;
    std::list<MetaData::Map> map_list;
    std::list<metatable::Snake> snake_meta_list;
    std::list<MetaData::Snake> snake_list;
    std::list<metatable::RankReward> rankreward_meta_list;
    std::list<MetaData::RankReward> rankreward_list;
    std::list<metatable::KillReward> killreward_meta_list;
    std::list<MetaData::KillReward> killreward_list;
    std::list<metatable::RankPoint> rankpoint_meta_list;
    std::list<MetaData::RankPoint> rankpoint_list;
    std::list<metatable::KillPoint> killpoint_meta_list;
    std::list<MetaData::KillPoint> killpoint_list;
    std::list<metatable::Robot> robot_meta_list;
    std::vector<MetaData::Robot> robot_list;

    std::map<std::string, MetaData::Parameter*> parameter_hash;
    std::map<int, MetaData::Map*> gamemap_hash;
    std::map<int, MetaData::Snake*> snake_hash;
    std::map<int, MetaData::RankReward*> rankreward_hash;
    std::map<int, MetaData::KillReward*> killreward_hash;
    std::map<int, MetaData::RankPoint*> rankpoint_hash;
    std::map<int, MetaData::KillPoint*> killpoint_hash;
    std::map<int, MetaData::Robot*> robot_hash;

    void Load()
    {
        if (!f8::IsOnlineEnv()) {
            res_path = a8::Format("../../../conf_test/game%d/gameserver/res/",
                                  {
                                   GAME_ID,
                                  });
        } else {
            res_path = "../res/";
        }

        f8::ReadCsvMetaFile(res_path + "parameter@parameter.csv", parameter_meta_list);
        f8::ReadCsvMetaFile(res_path + "map@map.csv", map_meta_list);
        f8::ReadCsvMetaFile(res_path + "snake@snake.csv", snake_meta_list);
        f8::ReadCsvMetaFile(res_path + "rankReward@rankReward.csv", rankreward_meta_list);
        f8::ReadCsvMetaFile(res_path + "killReward@killReward.csv", killreward_meta_list);
        f8::ReadCsvMetaFile(res_path + "killPoint@killPoint.csv", killpoint_meta_list);
        f8::ReadCsvMetaFile(res_path + "rankPoint@rankPoint.csv", rankpoint_meta_list);
        f8::ReadCsvMetaFile(res_path + "robot@robot.csv", robot_meta_list);
        BindToMetaData();
        #if 1
        {
            MetaMgr::Instance()->gas_inactive_time = MetaMgr::Instance()->GetSysParamAsInt("gas_inactive_time");
            MetaMgr::Instance()->jump_time = MetaMgr::Instance()->GetSysParamAsFloat("jump_time");
            MetaMgr::Instance()->kill_param = MetaMgr::Instance()->GetSysParamAsFloat("kill_parameter");
            MetaMgr::Instance()->rank_param = MetaMgr::Instance()->GetSysParamAsFloat("rank_parameter");
            MetaMgr::Instance()->fighting_mode = MetaMgr::Instance()->GetSysParamAsInt("fighting_mode", 1);
            MetaMgr::Instance()->tank_damage1 = MetaMgr::Instance()->GetSysParamAsInt("tank_damage1");
            MetaMgr::Instance()->tank_damage2 = MetaMgr::Instance()->GetSysParamAsInt("tank_damage2");
            MetaMgr::Instance()->tank_damage_interval = MetaMgr::Instance()->GetSysParamAsFloat("tank_damage_interval", 1);
            MetaMgr::Instance()->average_oil = MetaMgr::Instance()->GetSysParamAsInt("average_oil");
            MetaMgr::Instance()->max_oil = MetaMgr::Instance()->GetSysParamAsInt("max_oil");
        }
        #endif
    }

private:

    void BindToMetaData()
    {

        for (auto& meta : parameter_meta_list) {
            MetaData::Parameter& item = a8::FastAppend(parameter_list);
            item.p = &meta;
            item.Init();
            parameter_hash[item.p->param_name()] = &item;
        }

        {
            for (auto& meta : map_meta_list) {
                MetaData::Map& item = a8::FastAppend(map_list);
                item.p = &meta;
                item.Init();
                gamemap_hash[item.p->map_id()] = &item;
            }
        }

        {
            for (auto& meta : snake_meta_list) {
                MetaData::Snake& item = a8::FastAppend(snake_list);
                item.p = &meta;
                item.Init();
                snake_hash[item.p->snake_id()] = &item;
            }
        }

       for (auto& meta : rankreward_meta_list) {
            MetaData::RankReward& item = a8::FastAppend(rankreward_list);
            item.p = &meta;
            rankreward_hash[item.p->rank()] = &item;
        }

       for (auto& meta : killreward_meta_list) {
           MetaData::KillReward& item = a8::FastAppend(killreward_list);
           item.p = &meta;
           killreward_hash[item.p->kill_num()] = &item;
       }

       for (auto& meta : killpoint_meta_list) {
           MetaData::KillPoint& item = a8::FastAppend(killpoint_list);
           item.p = &meta;
           killpoint_hash[item.p->kill_num()] = &item;
       }

       for (auto& meta : rankpoint_meta_list) {
           MetaData::RankPoint& item = a8::FastAppend(rankpoint_list);
           item.p = &meta;
           rankpoint_hash[item.p->rank()] = &item;
       }

       for (auto& meta : robot_meta_list) {
           MetaData::Robot& item = a8::FastAppend(robot_list);
           item.p = &meta;
           item.Init();
           robot_hash[meta.id()] = &item;
       }

    }

private:
    std::string res_path;
};

void MetaMgr::Init()
{
    loader_ = new MetaDataLoader();
    loader_->Load();
}

void MetaMgr::UnInit()
{
    delete loader_;
    loader_ = nullptr;
}

void MetaMgr::Reload()
{
    delete loader_;
    loader_ = nullptr;

    loader_ = new MetaDataLoader();
    loader_->Load();
}

int MetaMgr::GetSysParamAsInt(const std::string& param_name, int def_val)
{
    auto itr = loader_->parameter_hash.find(param_name);
    return itr != loader_->parameter_hash.end() ? itr->second->int_val: def_val;
}

double MetaMgr::GetSysParamAsFloat(const std::string& param_name, double def_val)
{
    auto itr = loader_->parameter_hash.find(param_name);
    return itr != loader_->parameter_hash.end() ? itr->second->float_val: def_val;
}

std::string MetaMgr::GetSysParamAsString(const std::string& param_name, const char* def_val)
{
    auto itr = loader_->parameter_hash.find(param_name);
    return itr != loader_->parameter_hash.end() ? itr->second->str_val: def_val;
}

MetaData::Map* MetaMgr::GetMap(int map_id)
{
    auto itr = loader_->gamemap_hash.find(map_id);
    return itr != loader_->gamemap_hash.end() ? itr->second : nullptr;
}

MetaData::Snake* MetaMgr::GetSnake(int snake_id)
{
    auto itr = loader_->snake_hash.find(snake_id);
    return itr != loader_->snake_hash.end() ? itr->second : nullptr;
}

float MetaMgr::GetRankRewardParam(int rank)
{
    auto itr = loader_->rankreward_hash.find(rank);
    return itr != loader_->rankreward_hash.end() ? itr->second->p->parameter() : 0.0f;
}

float MetaMgr::GetKillRewardParam(int kill_num)
{
    auto itr = loader_->killreward_hash.find(kill_num);
    return itr != loader_->killreward_hash.end() ? itr->second->p->parameter() : 0.0f;
}

int MetaMgr::GetRankPointParam1(int rank)
{
    auto itr = loader_->rankpoint_hash.find(rank);
    return itr != loader_->rankpoint_hash.end() ? itr->second->p->parameter() : 0;
}

int MetaMgr::GetRankPointParam2(int rank)
{
    auto itr = loader_->rankpoint_hash.find(rank);
    return itr != loader_->rankpoint_hash.end() ? itr->second->p->parameter2() : 0;
}

int MetaMgr::GetKillPointParam1(int kill_num)
{
    auto itr = loader_->killpoint_hash.find(kill_num);
    return itr != loader_->killpoint_hash.end() ? itr->second->p->parameter() : 0;
}

int MetaMgr::GetKillPointParam2(int kill_num)
{
    auto itr = loader_->killpoint_hash.find(kill_num);
    return itr != loader_->killpoint_hash.end() ? itr->second->p->parameter2() : 0;
}

std::vector<MetaData::Robot>* MetaMgr::GetRobotList()
{
    return &loader_->robot_list;
}

MetaData::Robot* MetaMgr::GetRobot(int robot_id)
{
    auto itr = loader_->robot_hash.find(robot_id);
    return itr != loader_->robot_hash.end() ? itr->second : nullptr;
}
